﻿-- инициализация библиотеки
local lib = LibStub:NewLibrary ("LibFrame", 1)
local AceGUI = LibStub ("AceGUI-3.0")

local HF_BUTTON_TEXT_PADDING = 20
local HF_BUTTON_HEIGHT = 22
local HF_ROW_TEXT_PADDING = 5

-- получение фрэйма-родителя всех фрэймов (основной фрэйм)
function lib:Get_MainFrame (name, sizeX, sizeY, position)
	local HF_Frame_Main = CreateFrame ("Frame", name, UIParent)

	HF_Frame_Main:SetMovable (true)
	HF_Frame_Main:EnableMouse (true)
	HF_Frame_Main:SetScript ("OnMouseDown", function (self, button)
		if (HF_Save_Data.clearFramePosition == true) then return end
		if button == "LeftButton" and not self.isMoving then
			self:StartMoving ()
			self.isMoving = true;
		end
	end)
	HF_Frame_Main:SetScript ("OnMouseUp", function (self, button)
		if (HF_Save_Data.clearFramePosition == true) then return end
		if button == "LeftButton" and self.isMoving then
			self:StopMovingOrSizing ()
			self.isMoving = false;
		end
	end)
	HF_Frame_Main:SetScript ("OnHide", function (self)
		if (HF_Save_Data.clearFramePosition == true) then return end
		if (self.isMoving) then
			self:StopMovingOrSizing ()
			self.isMoving = false;
		end
	end)

	HF_Frame_Main:SetPoint (position)
	HF_Frame_Main:SetSize (sizeX, sizeY)

	return HF_Frame_Main
end

-- создание фрэйма для родителя
function lib:Get_Frame (name, parent, sizeX, sizeY, position)
	local frame = CreateFrame ("Frame", name, parent)
	frame:SetSize (sizeX, sizeY)
	frame:SetBackdrop (
	{
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = true,
		tileSize = 16,
		edgeSize = 16,
		insets = {
			left = 4,
			right = 4,
			top = 4,
			bottom = 4
		}
	})
	frame:SetBackdropColor (0, 0, 0)
	frame:SetPoint (position)
	return frame
end

-- создание поля с текстом
function lib:Get_Text (frame, size, text, font, Point, pX, pY, sX, sY, jH, jV)
	local tInfo = frame:CreateFontString()
	tInfo:SetFont (font, size)
	tInfo:SetText (text)
	if (jH ~= nil and jV ~= nil) then
		tInfo:SetJustifyH (jH)
		tInfo:SetJustifyV (jV)
	end
	tInfo:SetPoint (Point, pX, pY)
	if (sX ~= nil and sY ~= nil) then
		tInfo:SetSize (sX, sY)
	end

	return tInfo
end

function lib:Get_Texture_Button (h)
	h:SetHeight (24)

	h:SetHighlightTexture ("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight", "ADD")

	return h
end

function lib:newSlider (hiddenOptions, parent, cvar, minRange, maxRange, stepSize, defaultValue, Widgets, setValue)
	local cvarTable = hiddenOptions[cvar] or {}
	local label = cvarTable['prettyName'] or cvar
	local description = cvarTable['description'] or ''

	description = description .. '\n\nЗначение по дефолту: ' .. (defaultValue or '')
	local slider = CreateFrame ('Slider', 'HF_Slider' .. cvar, parent, 'OptionsSliderTemplate')

	slider.cvar = cvar
	slider.SetCVarValue = setValue or sliderSetCVar
	slider:SetScript ('OnShow', sliderRefresh)
	stepSize = stepSize or 1
	slider:SetValueStep (stepSize)
	slider:SetObeyStepOnDrag (true)

	slider:SetMinMaxValues (minRange, maxRange)
	slider.minText = _G[slider:GetName () .. 'Low']
	slider.maxText = _G[slider:GetName () .. 'High']
	slider.text = _G[slider:GetName () .. 'Text']
	slider.minText:SetText (minRange)
	slider.maxText:SetText (maxRange)
	slider.text:SetText (label)

	local valueBox = CreateFrame ('editbox', nil, slider)
	valueBox:SetPoint ('TOP', slider, 'BOTTOM', 0, 0)
	valueBox:SetSize (60, 14)
	valueBox:SetFontObject (GameFontHighlightSmall)
	valueBox:SetAutoFocus (false)
	valueBox:SetJustifyH ('CENTER')
	valueBox:SetScript ('OnEscapePressed', function (self)
		-- ignore input, reset value to current cvar
		local current, default = GetCVarInfo (slider.cvar)
		self:SetText (current or default)
		self:ClearFocus ()
	end)
	valueBox:SetScript ('OnEnterPressed', function (self)
		local current, default = GetCVarInfo (slider.cvar)
		local value = tonumber (self:GetText ()) or current or default
		local factor = 1 / stepSize
		value = floor (value * factor + 0.5) / factor
		value = max (minRange, min (maxRange, value))
		slider:SetValue (value)
		self:SetText (value)
		self:ClearFocus ()
	end)
	slider:HookScript ('OnValueChanged', function (self, value)
		local factor = 1 / stepSize
		value = floor (value * factor + 0.5) / factor
		valueBox:SetText (value)
	end)
	valueBox:SetScript ('OnChar', function (self) -- filter input to decimal values
		self:SetText (self:GetText ():gsub ('[^%.0-9]+', ''):gsub ('(%..*)%.', '%1'))
	end)
	valueBox:SetMaxLetters (5)

	valueBox:SetBackdrop ( {
		bgFile = 'Interface/ChatFrame/ChatFrameBackground',
		edgeFile = 'Interface/ChatFrame/ChatFrameBackground',
		tile = true, edgeSize = 1, tileSize = 5,
	})
	valueBox:SetBackdropColor (0, 0, 0, 0.5)
	valueBox:SetBackdropBorderColor (0.3, 0.3, 0.3, 0.8)

	slider.valueBox = valueBox

	slider:HookScript ('OnValueChanged', slider.SetCVarValue)

	slider.tooltipText = label
	slider.tooltipRequirement = description

	Widgets[slider] = cvar
	return slider
end

function lib:Get_DropBox (name, parent, sortMode, pointType, pointX, pointY, func)
	local ddl = CreateFrame("Frame", name, parent, "UIDropDownMenuTemplate")
	ddl:SetPoint (pointType, parent, "TOPLEFT", pointX, pointY)
	ddl.initialize = function(dropdown)
		local sortMode = sortMode
		for i, mode in next, sortMode do
			local info = UIDropDownMenu_CreateInfo()
			info.text = sortMode[i]
			info.value = sortMode[i]
			info.func = func
			UIDropDownMenu_AddButton(info)
		end
		UIDropDownMenu_SetSelectedValue (dropdown, parent)
	end
	ddl:HookScript("OnShow", ddl_Size_my_character.initialize)
end

function lib:CreateTable(parent, texts, widths, justfiesH, rightPadding)
	assert(#texts == #widths and #texts == #justfiesH,
	"All specification tables must be the same size; texts = "..tostring(#texts).. " - widths = "..tostring(#widths).. " - justfiesH = "..tostring(#justfiesH))
	-- Compute widths
	local totalFixedWidths = rightPadding or 0
	local numDynamicWidths = 0
	for i,w in ipairs(widths) do
		if w > 0 then
			totalFixedWidths = totalFixedWidths + w
		else
			numDynamicWidths = numDynamicWidths + 1
		end
	end
	local remainingWidthSpace = parent:GetWidth() - totalFixedWidths
	assert(remainingWidthSpace >= 0, "Указанная ширина "..remainingWidthSpace.." превышает пороговую ширину "..parent:GetWidth().." "..totalFixedWidths)

	local dynamicWidth = math.floor(remainingWidthSpace / numDynamicWidths)
	local leftoverWidth = remainingWidthSpace % numDynamicWidths
	for i,w in ipairs(widths) do
		if w <= 0 then
			numDynamicWidths = numDynamicWidths - 1
			if numDynamicWidths then
				widths[i] = dynamicWidth
			else
				widths[i] = dynamicWidth + leftoverWidth
			end
		end
	end

	-- Make headers
	parent.headers = {}
	for i=1,#texts do
		local text, width, justifyH = texts[i], widths[i], justfiesH[i]
		local h = lib:CreateTableHeader(parent, text, width)
		h:SetNormalFontObject("GameFontHighlightSmall")
		h:SetText(text)
		h:GetFontString():SetJustifyH(justifyH)
		h:SetWidth(width)
		if #parent.headers == 0 then
			h:SetPoint("TOPLEFT")
		else
			h:SetPoint("TOPLEFT", parent.headers[#parent.headers], "TOPRIGHT")
		end
		table.insert(parent.headers, h)
	end

	-- Make a frame for the rows
	local rowFrame = CreateFrame("Frame", nil, parent)
	rowFrame:SetPoint("TOP", parent.headers[#parent.headers], "BOTTOM")
	rowFrame:SetPoint("BOTTOMLEFT")
	rowFrame:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", -rightPadding, 0)
	parent.rowFrame = rowFrame

	-- Compute number of rows
	local fontHeight = 13 + 2
	local rowHeight = fontHeight + 4
	rowFrame.rowHeight = rowHeight
	local numRows = math.floor(rowFrame:GetHeight() / rowHeight)

	-- Make rows
	rowFrame.rows = {}
	for i=1,numRows do
		local r = lib:CreateTableRow(rowFrame, rowHeight, widths, justfiesH)
		if #rowFrame.rows == 0 then
			r:SetPoint("TOP")
		else
			r:SetPoint("TOP", rowFrame.rows[#rowFrame.rows], "BOTTOM")
		end
		table.insert(rowFrame.rows, r)
	end
end

function lib:CreateTableHeader(parent)
	local h = CreateFrame("Button", nil, parent)
	h:SetHeight(24)

	local tl = h:CreateTexture(nil, "BACKGROUND")
	tl:SetTexture("Interface\\FriendsFrame\\WhoFrame-ColumnTabs")
	tl:SetWidth(5)
	tl:SetHeight(24)
	tl:SetPoint("TOPLEFT")
	tl:SetTexCoord(0, 0.07815, 0, 0.75)

	local tr = h:CreateTexture(nil, "BACKGROUND")
	tr:SetTexture("Interface\\FriendsFrame\\WhoFrame-ColumnTabs")
	tr:SetWidth(5)
	tr:SetHeight(24)
	tr:SetPoint("TOPRIGHT")
	tr:SetTexCoord(0.90625, 0.96875, 0, 0.75)

	local tm = h:CreateTexture(nil, "BACKGROUND")
	tm:SetTexture("Interface\\FriendsFrame\\WhoFrame-ColumnTabs")
	tm:SetHeight(24)
	tm:SetPoint("LEFT", tl, "RIGHT")
	tm:SetPoint("RIGHT", tr, "LEFT")
	tm:SetTexCoord(0.07815, 0.90625, 0, 0.75)

	h:SetHighlightTexture(
	"Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight", "ADD")

	return h
end

function lib:CreateTableRow(parent, rowHeight, widths, justifiesH)
	local row = CreateFrame("Button", nil, parent)
	row:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight", "ADD")
	row:SetHeight(rowHeight)
	row:SetPoint("LEFT")
	row:SetPoint("RIGHT")

	row.cells = {}
	for i,w in ipairs(widths) do
		local c = row:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
		c:SetHeight(rowHeight)
		c:SetWidth(w - (2 * HF_ROW_TEXT_PADDING))
		c:SetJustifyH(justifiesH[i])
		if #row.cells == 0 then
			c:SetPoint("LEFT", row, "LEFT", HF_ROW_TEXT_PADDING, 0)
		else
			c:SetPoint("LEFT", row.cells[#row.cells], "RIGHT", 2 * HF_ROW_TEXT_PADDING, 0)
		end
		table.insert(row.cells, c)
		c:SetText(w)
	end

	return row
end